# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

from unittest import TestCase

from jubakit.loader.odbc import ODBCLoader

class ODBCLoaderTest(TestCase):
  def test_simple(self):
    loader = ODBCLoader()
