# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

from ..base import BaseLoader
from ..compat import *

class ODBCLoader(BaseLoader):
  """
  Loader to process ODBC data sources.
  """

  # TODO not implemented yet.
  pass
